@@ -1,7 +1,7 @@
 package App::metasyn;
 
-our $DATE = '2017-02-24'; # DATE
-our $VERSION = '0.003'; # VERSION
+our $DATE = '2017-02-27'; # DATE
+our $VERSION = '0.004'; # VERSION
 
 use 5.010001;
 use strict;
@@ -49,6 +49,11 @@ _
             argv => [qw/-l/],
             'x.doc.max_result_lines' => 10,
         },
+        {
+            summary => 'List all installed themes, along with all their categories',
+            argv => [qw/-l -c/],
+            'x.doc.max_result_lines' => 10,
+        },
         {
             summary => 'List all names from a theme',
             argv => [qw/foo/],
@@ -61,7 +66,7 @@ _
         },
         {
             summary => 'List all categories from a theme',
-            argv => [qw(christmas --categories)],
+            argv => [qw(christmas -c)],
             'x.doc.max_result_lines' => 10,
         },
     ],
@@ -77,8 +82,27 @@ sub metasyn {
 
     my $action = $args{action};
 
-    return [200, "OK", [Acme::MetaSyntactic->new->themes]]
-        if $action eq 'list-themes';
+    if ($action eq 'list-themes') {
+        my @res;
+        for my $th (Acme::MetaSyntactic->new->themes) {
+            if ($args{categories}) {
+                my $pkg = "Acme::MetaSyntactic::$th";
+                (my $pkg_pm = "$pkg.pm") =~ s!::!/!g;
+                return [500, "Can't load $pkg: $@"]
+                    unless (eval { require $pkg_pm; 1 });
+                my @cats;
+                @cats = $pkg->categories if $pkg->can("categories");
+                if (@cats) {
+                    push @res, "$th/$_" for @cats;
+                } else {
+                    push @res, $th;
+                }
+            } else {
+                push @res, $th;
+            }
+        }
+        return [200, "OK", \@res];
+    }
 
     my $theme = $args{theme};
     return [400, "Please specify theme"] unless $theme;
@@ -127,7 +151,7 @@ App::metasyn - Alternative front-end to Acme::MetaSyntactic
 
 =head1 VERSION
 
-This document describes version 0.003 of App::metasyn (from Perl distribution App-metasyn), released on 2017-02-24.
+This document describes version 0.004 of App::metasyn (from Perl distribution App-metasyn), released on 2017-02-27.
 
 =head1 SYNOPSIS
 
@@ -162,7 +186,7 @@ Result:
      "afke",
      "alice",
      "alphabet",
- # ...snipped for brevity...
+ # ...snipped 130 lines for brevity...
      "viclones",
      "wales_towns",
      "weekdays",
@@ -172,6 +196,30 @@ Result:
    {},
  ]
 
+=item * List all installed themes, along with all their categories:
+
+ metasyn( action => "list-themes", categories => 1);
+
+Result:
+
+ [
+   200,
+   "OK",
+   [
+     "abba",
+     "afke",
+     "alice",
+     "alphabet/la",
+ # ...snipped 2148 lines for brevity...
+     "zodiac/Vedic",
+     "zodiac/Western",
+     "zodiac/Western/Traditional",
+     "zodiac/Chinese",
+     "zodiac/Western/Real",
+   ],
+   {},
+ ]
+
 =item * List all names from a theme:
 
  metasyn( theme => "foo");
@@ -186,7 +234,7 @@ Result:
      "bar",
      "baz",
      "foobar",
- # ...snipped for brevity...
+ # ...snipped 37 lines for brevity...
      "weide",
      "does",
      "hok",
@@ -206,16 +254,16 @@ Result:
    200,
    "OK",
    [
-     "opneslae",
-     "bushy",
-     "snowball",
      "upatree",
- # ...snipped for brevity...
-     "evergreen",
-     "wunorse",
-     "pepper",
-     "minstix",
+     "opneslae",
+     "sugarplum",
+     "bushy",
+ # ...snipped 3 lines for brevity...
      "shinny",
+     "minstix",
+     "wunorse",
+     "mary",
+     "pepper",
    ],
    {},
  ]
@@ -226,7 +274,7 @@ Result:
 
 Result:
 
- [200, "OK", ["santa", "elf", "snowman", "reindeer"], {}]
+ [200, "OK", ["reindeer", "santa", "snowman", "elf"], {}]
 
 =back
 
